#! /usr/bin/env node
/* eslint-disable @typescript-eslint/no-explicit-any */
"use strict";

var _fs = _interopRequireDefault(require("fs"));
var _tokensTemplate = require("./tokens.template.js");
var _themeTemplate = require("./theme.template.js");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const args = process.argv;
const ignoreObjectProperties = ['type', 'value'];
const ignoreLeafFields = ['type', 'description'];
const auxTheme = {};
const object = {};
const shadows = {};
const toCamelCase = (unformattedWord = '') => {
  if (!unformattedWord.includes('-') || unformattedWord.startsWith('-') && unformattedWord.match(/[0-9]+/g)) return unformattedWord;
  return unformattedWord.toLowerCase().replace(/[^a-zA-Z0-9{]+(.)/g, (_, chr) => chr.toUpperCase());
};
const format = (word = '') => {
  const ignoreObjectPropertiesFormatted = ignoreObjectProperties.map(prop => `${prop}\\.+`);
  const propertiesToIgnoreRegex = new RegExp(ignoreObjectPropertiesFormatted.join('|'), 'gi');
  const formattedWord = word.replace(propertiesToIgnoreRegex, '').replace(/\.value$/gi, '').split('.').map(dotSeparatedWord => toCamelCase(dotSeparatedWord)).join('.');
  return formattedWord;
};
const stringToObject = (key = '', value) => {
  const propertiesObject = key.split('.').reduceRight((acc, currentVal) => {
    return {
      [currentVal]: acc
    };
  }, value);
  return propertiesObject;
};
const mergeRecursive = (object1, object2) => {
  for (const p in object2) {
    try {
      if (object2[p].constructor == Object) {
        object1[p] = mergeRecursive(object1[p], object2[p]);
      } else {
        object1[p] = object2[p];
      }
    } catch (e) {
      object1[p] = object2[p];
    }
  }
  return object1;
};
const generateTheme = (currentObject, property = '') => {
  if (typeof currentObject === 'string') {
    if (ignoreObjectProperties.includes(currentObject)) {
      return;
    }
    const formattedProperty = format(property);
    const formattedValue = format(currentObject);
    auxTheme[formattedProperty] = formattedValue;
    return;
  }
  return Object.keys(currentObject).map(currentObjectKey => {
    if (ignoreLeafFields.includes(currentObjectKey) && typeof currentObject[currentObjectKey] === 'string') {
      return;
    }
    return generateTheme(currentObject[currentObjectKey], property ? `${property}.${currentObjectKey}` : currentObjectKey);
  }).join('');
};
const saveFile = (dir, object, generateFunction, generateIndexFunction) => {
  const pathToSave = args[4];
  const keys = Object.keys(object);
  const dirPath = `${pathToSave}/${dir}`;
  keys.forEach(key => {
    if (!_fs.default.existsSync(dirPath)) {
      _fs.default.mkdirSync(dirPath, {
        recursive: true
      });
    }
    _fs.default.writeFileSync(`${dirPath}/${key}.ts`, generateFunction(key, stringToReferenceObject(JSON.stringify(object[key])), keys));
  });
  _fs.default.writeFileSync(`${dirPath}/index.ts`, generateIndexFunction(keys));
};
const stringToReferenceObject = (object = '') => {
  return object.replace(/"\{|\}"/g, '');
};
const formatEffectString = object => {
  return `${object.x} ${object.y} ${object.blur} ${object.spread} ${object.color}`;
};
const formatEffects = object => {
  Object.keys(object).map(elevation => {
    Object.keys(elevation).map(() => {
      if (isNaN(parseInt(Object.keys(object[elevation])[0]))) {
        shadows[elevation] = formatEffectString(object[elevation]);
        return;
      }
      shadows[elevation] = Object.entries(object[elevation]).map(value => formatEffectString(value[1])).join(', ');
    });
  });
  return shadows;
};
(async function run() {
  try {
    const pathToTokensFile = args[3];
    const file = await (specifier => new Promise(r => r(`${specifier}`)).then(s => _interopRequireWildcard(require(s))))(pathToTokensFile);
    Object.keys(file).filter(key => !key.startsWith('$')).map(key => generateTheme(file[key]));
    Object.keys(auxTheme).map(property => {
      mergeRecursive(object, stringToObject(property, auxTheme[property]));
    });
    const tokens = {
      color: object.ref.color,
      effect: {
        elevation: formatEffects(object.ref.elevation)
      },
      spacing: {
        padding: object.ref.padding,
        flexbox: object.ref.flexbox,
        borderRadius: object.ref.borderRadius,
        borderWidth: object.ref.borderWidth
      },
      typography: {
        fontFamily: object.ref.fontFamily,
        fontWeight: object.ref.fontWeight,
        letterSpacing: object.ref.letterSpacing,
        fontSize: object.ref.fontSize,
        lineHeight: object.ref.lineHeight,
        textDecoration: object.ref.textDecoration,
        textCase: object.ref.textCase,
        paragraphSpacing: object.ref.paragraphSpacing
      }
    };
    const theme = {
      colors: {
        content: object.sys.content,
        contentState: object.sys.contentState,
        background: object.sys.background,
        backgroundState: object.sys.backgroundState,
        border: object.sys.border,
        borderState: object.sys.borderState
      },
      effects: {
        shadow: object.sys.boxShadow
      },
      spacing: {
        borderRadius: object.sys.borderRadius
      },
      typography: object.sys.typography,
      components: object.comp
    };
    saveFile('tokens', tokens, _tokensTemplate.generateTokenTemplate, _tokensTemplate.generateTokenIndexFile);
    saveFile('theme', theme, _themeTemplate.generateThemeTemplate, _themeTemplate.generateThemeIndexFile);
    console.log('Theme generated successfully.');
  } catch (error) {
    console.error('Error while generating the theme: ', error);
  }
})();